/* The 36057 is usually the companion of the 36060 MJPEG codec, but even though it doesn't 
   exist in e.g. the RRG/Marvel, it is a pretty good thing to emulate it (since we get source
   code compatibility with the Iomega Buz driver, and the mga struct has all its 
   JPEG memory buffer stuff in one special struct) */

/*
 *   Set the registers for the size we have specified. Don't bother
 *   trying to understand this without the ZR36060 manual in front of
 *   you [AC].
 *
 *   PS: The manual is free for download in .pdf format from
 *   www.zoran.com - nicely done those folks.
 */

#ifndef _ZR36057_H_
#define _ZR36057_H_

#define MJPEG_NUM_STAT_COM    4
#define MJPEG_MASK_STAT_COM   3

#define MJPEG_MAX_FRAME     256  /* Must be a power of 2 */
#define MJPEG_MASK_FRAME    255  /* Must be MJPEG_MAX_FRAME-1 */

/* The ZR36057 only supports a maximum width of 720, but some V4L
   applications (e.g. xawtv are more happy with 768).
   If XAWTV_HACK is defined, we try to fake a device with bigger width */

#define XAWTV_HACK

#ifdef XAWTV_HACK
#define   OVL_MAX_WIDTH   768   /* never display more than 768 pixels */
#else
#define   OVL_MAX_WIDTH   720   /* never display more than 720 pixels */
#endif
#define   OVL_MAX_HEIGHT  576   /* never display more than 576 rows */
#define   OVL_MIN_WIDTH    32   /* never display less than 32 pixels */
#define   OVL_MIN_HEIGHT   24   /* never display less than 24 rows */

/* Defining the overlay mask (used for clipping the overlay window, hence the name) */
struct zr36057_video_clip
{
	__s32	x,y;
	__s32	width, height;
	struct	zr36057_video_clip *next;	/* For user use/driver use only */
};

enum zr36057_buffer_state
{
   MJPEG_STATE_USER,                /* buffer is owned by application */
   MJPEG_STATE_PEND,                /* buffer is queued in pend[] ready to feed to I/O */
   MJPEG_STATE_DMA,                 /* buffer is queued in dma[] for I/O */
   MJPEG_STATE_DONE                 /* buffer is ready to return to application */
};

struct zr36057_sync
{
   unsigned long frame;      /* Frame (0 - n) for double buffer */
   unsigned long length;     /* number of code bytes in buffer (capture only) */
   unsigned long seq;        /* frame sequence number */
   struct timeval timestamp; /* timestamp */
};

struct zr36057_gbuffer
{
   u32 * frag_tab;                /* addresses of frag table */
   u32 frag_tab_bus;              /* same value cached to save time in ISR */
   enum zr36057_buffer_state state;   /* non-zero if corresponding buffer is in use in grab queue */
   struct zr36057_sync bs;            /* info to return to application */
};

enum zr36057_mode
{
  FAKE_ZR36057,
  REAL_ZR36057
};

/* These were copied from videodev.h - if they change, change here accordingly */
#define VIDEO_PALETTE_RGB565	3	/* 565 16 bit RGB */
#define VIDEO_PALETTE_RGB24	4	/* 24bit RGB */
#define VIDEO_PALETTE_RGB32	5	/* 32bit RGB */	
#define VIDEO_PALETTE_RGB555	6	/* 555 15bit RGB */
#define VIDEO_PALETTE_YUV422	7	/* YUV422 capture */

struct zr36057_dev
{
  enum zr36057_mode realchip; 

  /* MJPEG API (user app communication) parameters */
  unsigned long nbufs;             /* Number of buffers */
  unsigned long bufsize;           /* Size of buffers in bytes */
  int buffers_allocated;           /* Flag if buffers are allocated  */
  int need_contiguous;             /* Flag if contiguous buffers are needed */

#if LINUX_VERSION_CODE >= 0x020300
   wait_queue_head_t   capq;      /* wait here for grab to finish */
#else
   struct wait_queue * capq;      /* wait here for grab to finish */
#endif
  
  /* grab queue counts/indices, mask with MJPEG_MASK_STAT_COM before using as index */
  /* (dma_head - dma_tail) is number active in DMA, must be <= MJPEG_NUM_STAT_COM */
  /* (value & MJPEG_MASK_STAT_COM) corresponds to index in stat_com table */
  unsigned long que_head;     /* Index where to put next buffer which is queued */
  unsigned long dma_head;     /* Index of next buffer which goes into stat_com  */
  unsigned long dma_tail;     /* Index of last buffer in stat_com               */
  unsigned long que_tail;     /* Index of last buffer in queue                  */
  unsigned long seq_num;      /* count of frames since grab/play started */

  /* zr36057's code buffer table */
  u32 * stat_com;            /* stat_com[i] is indexed by dma_head/tail & MJPEG_MASK_STAT_COM */

  /* fake ZR36057 internal registers */
  int fake_stat_com_count; /* used for counting through the pics in fake ZR36057 mode */
  int fake_frag_num; /* pointing to the currently worked on fragment in stat_com[i] */
  int fake_frag_done; /* showing the "fill status" of the current worked on fragment */
  int fake_seq_num; /* A frame sequence number (modulo 256) in compression mode */
  int fake_rec_bytes; /* Counting the recorded bytes in compression mode */

  /* (value & MJPEG_MASK_FRAME) corresponds to index in pend[] queue */
  int pend[MJPEG_MAX_FRAME];

  /* array indexed by frame number */
  struct zr36057_gbuffer gbuf[MJPEG_MAX_FRAME];   /* MJPEG user app buffers' info */

  /* These routines are used in zr36057_qbuf_ops, the first enables and 
     the second starts the JPEG code transfers */
  void (*mjpeg_enable)(void *cardstruct, int mode);
  void (*mjpeg_go)(void *cardstruct, int mode);

  /* these variables are just used when the zr36057 is "for real" (realchip = 1) */
  /* they are set in zr36057_init */
  void (*or)(int value, int reg);
  void (*and)(int value, int reg);
  void (*write)(int value, int reg);

  /* These variables are only needed in real mode */
  int video_interlace;
  u32 *overlay_mask;
  int triton;
};

int zr36057_init(struct zr36057_dev *zr36057, enum zr36057_mode real_chip, int triton,
		 void *orfunction, void * andfunction, void * writefunction,
		 void * mjpeg_enable_function, void * mjpeg_go_function);
int zr36057_sync(struct zr36057_dev *zr36057, struct zr36060_dev *zr36060, 
		 struct zr36057_sync *bs, spinlock_t *lock);
int zr36057_queuebuf_ops(struct zr36057_dev *zr36057, struct zr36060_dev *zr36060, 
			 u32 frame, int mode, void *hostcard);
int zr36057_fbuffer_alloc(struct zr36057_dev *zr36057);
void zr36057_fbuffer_free(struct zr36057_dev * zr36057);
void zr36057_set_vfe(struct zr36057_dev *zr36057,
		     struct zr36060_dev *zr36060,
		     int video_width, int video_height, int windowwidth,
		     unsigned int video_format);
void zr36057_write_overlay_mask(struct zr36057_dev * zr36057, 
				struct zr36057_video_clip * vp, int count,
				int windowwidth, int windowheight);
void zr36057_set_jpg(struct zr36057_dev * zr36057, struct zr36060_dev *zr36060, int mode);
int zr36057_mmap(struct zr36057_dev *zr36057, const char * adr, unsigned long size);
void zr36057_feed_stat_com(struct zr36057_dev *zr36057, struct zr36060_dev *zr36060);
void zr36057_reap_stat_com(struct zr36057_dev *zr36057, struct zr36060_dev *zr36060);

/* Zoran ZR36057 registers */

#define ZR36057_VFEHCR          0x000   /* Video Front End, Horizontal Configuration Register */
#define ZR36057_VFEHCR_HSPol            (1<<30)
#define ZR36057_VFEHCR_HStart           10
#define ZR36057_VFEHCR_HEnd	        0
#define ZR36057_VFEHCR_Hmask	        0x3ff

#define ZR36057_VFEVCR          0x004   /* Video Front End, Vertical Configuration Register */
#define ZR36057_VFEVCR_VSPol            (1<<30)
#define ZR36057_VFEVCR_VStart           10
#define ZR36057_VFEVCR_VEnd	        0
#define ZR36057_VFEVCR_Vmask	        0x3ff

#define ZR36057_VFESPFR         0x008   /* Video Front End, Scaler and Pixel Format Register */
#define ZR36057_VFESPFR_ExtFl           (1<<26)
#define ZR36057_VFESPFR_TopField        (1<<25)
#define ZR36057_VFESPFR_VCLKPol         (1<<24)
#define ZR36057_VFESPFR_HFilter         21
#define ZR36057_VFESPFR_HorDcm          14
#define ZR36057_VFESPFR_VerDcm          8
#define ZR36057_VFESPFR_DispMode        6
#define ZR36057_VFESPFR_YUV422          (0<<3)
#define ZR36057_VFESPFR_RGB888          (1<<3)
#define ZR36057_VFESPFR_RGB565          (2<<3)
#define ZR36057_VFESPFR_RGB555          (3<<3)
#define ZR36057_VFESPFR_ErrDif          (1<<2)
#define ZR36057_VFESPFR_Pack24          (1<<1)
#define ZR36057_VFESPFR_LittleEndian    (1<<0)

#define ZR36057_VDTR            0x00c   /* Video Display "Top" Register */

#define ZR36057_VDBR            0x010   /* Video Display "Bottom" Register */

#define ZR36057_VSSFGR          0x014   /* Video Stride, Status, and Frame Grab Register */
#define ZR36057_VSSFGR_DispStride       16
#define ZR36057_VSSFGR_VidOvf           (1<<8)
#define ZR36057_VSSFGR_SnapShot         (1<<1)
#define ZR36057_VSSFGR_FrameGrab        (1<<0)

#define ZR36057_VDCR            0x018   /* Video Display Configuration Register */
#define ZR36057_VDCR_VidEn              (1<<31)
#define ZR36057_VDCR_MinPix             24
#define ZR36057_VDCR_Triton             (1<<24)
#define ZR36057_VDCR_VidWinHt           12
#define ZR36057_VDCR_VidWinWid          0

#define ZR36057_MMTR            0x01c   /* Masking Map "Top" Register */

#define ZR36057_MMBR            0x020   /* Masking Map "Bottom" Register */

#define ZR36057_OCR             0x024   /* Overlay Control Register */
#define ZR36057_OCR_OvlEnable           (1 << 15)
#define ZR36057_OCR_MaskStride          0

#define ZR36057_SPGPPCR         0x028   /* System, PCI, and General Purpose Pins Control Register */
#define ZR36057_SPGPPCR_SoftReset	(1<<24)

#define ZR36057_GPPGCR1         0x02c   /* General Purpose Pins and GuestBus Control Register (1) */

#define ZR36057_MCSAR           0x030   /* MPEG Code Source Address Register */

#define ZR36057_MCTCR           0x034   /* MPEG Code Transfer Control Register */
#define ZR36057_MCTCR_CodTime           (1 << 30)
#define ZR36057_MCTCR_CEmpty            (1 << 29)
#define ZR36057_MCTCR_CFlush            (1 << 28)
#define ZR36057_MCTCR_CodGuestID	20
#define ZR36057_MCTCR_CodGuestReg	16

#define ZR36057_MCMPR           0x038   /* MPEG Code Memory Pointer Register */

#define ZR36057_ISR             0x03c   /* Interrupt Status Register */
#define ZR36057_ISR_GIRQ1               (1<<30)
#define ZR36057_ISR_GIRQ0               (1<<29)
#define ZR36057_ISR_CodRepIRQ           (1<<28)
#define ZR36057_ISR_JPEGRepIRQ          (1<<27)

#define ZR36057_ICR             0x040   /* Interrupt Control Register */
#define ZR36057_ICR_GIRQ1               (1<<30)
#define ZR36057_ICR_GIRQ0               (1<<29)
#define ZR36057_ICR_CodRepIRQ           (1<<28)
#define ZR36057_ICR_JPEGRepIRQ          (1<<27)
#define ZR36057_ICR_IntPinEn            (1<<24)

#define ZR36057_I2CBR           0x044   /* I2C Bus Register */
#define ZR36057_I2CBR_SDA       	(1<<1)
#define ZR36057_I2CBR_SCL       	(1<<0)

#define ZR36057_JMC             0x100   /* JPEG Mode and Control */
#define ZR36057_JMC_JPG                 (1 << 31)
#define ZR36057_JMC_JPGExpMode          (0 << 29)
#define ZR36057_JMC_JPGCmpMode          (1 << 29)
#define ZR36057_JMC_MJPGExpMode         (2 << 29)
#define ZR36057_JMC_MJPGCmpMode         (3 << 29)
#define ZR36057_JMC_RTBUSY_FB           (1 << 6)
#define ZR36057_JMC_Go_en               (1 << 5)
#define ZR36057_JMC_SyncMstr            (1 << 4)
#define ZR36057_JMC_Fld_per_buff        (1 << 3)
#define ZR36057_JMC_VFIFO_FB            (1 << 2)
#define ZR36057_JMC_CFIFO_FB            (1 << 1)
#define ZR36057_JMC_Stll_LitEndian      (1 << 0)

#define ZR36057_JPC             0x104   /* JPEG Process Control */
#define ZR36057_JPC_P_Reset             (1 << 7)
#define ZR36057_JPC_CodTrnsEn           (1 << 5)
#define ZR36057_JPC_Active              (1 << 0)

#define ZR36057_VSP             0x108   /* Vertical Sync Parameters */
#define ZR36057_VSP_VsyncSize           16
#define ZR36057_VSP_FrmTot              0

#define ZR36057_HSP             0x10c   /* Horizontal Sync Parameters */
#define ZR36057_HSP_HsyncStart          16
#define ZR36057_HSP_LineTot             0

#define ZR36057_FHAP            0x110   /* Field Horizontal Active Portion */
#define ZR36057_FHAP_NAX                16
#define ZR36057_FHAP_PAX                0

#define ZR36057_FVAP            0x114   /* Field Vertical Active Portion */
#define ZR36057_FVAP_NAY                16
#define ZR36057_FVAP_PAY                0

#define ZR36057_FPP             0x118   /* Field Process Parameters */
#define ZR36057_FPP_Odd_Even            (1 << 0)

#define ZR36057_JCBA            0x11c   /* JPEG Code Base Address */

#define ZR36057_JCFT            0x120   /* JPEG Code FIFO Threshold */

#define ZR36057_JCGI            0x124   /* JPEG Codec Guest ID */
#define ZR36057_JCGI_JPEGuestID         4
#define ZR36057_JCGI_JPEGuestReg        0

#define ZR36057_GCR2            0x12c   /* GuestBus Control Register (2) */

#define ZR36057_POR             0x200   /* Post Office Register */
#define ZR36057_POR_POPen               (1<<25)
#define ZR36057_POR_POTime              (1<<24)
#define ZR36057_POR_PODir               (1<<23)

#define ZR36057_STR             0x300   /* "Still" Transfer Register */

#endif




